classdef ManualCluster < handle
    properties (Constant = true)
        cpus_per_node = 4
    end
    
    properties (SetAccess = private)
        node_numbers
        file_names
        node_of_job
    end

    properties (Dependent = true)
        max_num_jobs
        num_running_jobs
    end

    methods
        function obj = ManualCluster()
            obj.node_numbers = GridEngine.get_allocated_nodes();
            obj.file_names = '';
            obj.node_of_job = '';
        end
        
        function str = char(obj)
            str = 'node_numbers:';
            for i = 1:length(obj.node_numbers)
                str = [str ' ' char(obj.node_numbers(i))];      
            end
            str = [str char(13)];
            
            str = [str 'file_names:' char(13)];
            s = size(obj.file_names);
            for i = 1:s(1)
                str = [str char(obj.file_names(i,:)) char(13)];
            end
            
            str = [str 'node_of_job:' char(13)];
            t = size(obj.node_of_job);
            for i = 1:t(1)
                str = [str char(obj.node_of_job(i,:)) char(13)];
            end
        end

        function disp(obj)
            disp(char(obj))
        end
        
        function max_num_jobs = get.max_num_jobs(obj)
            max_num_jobs = length(obj.node_numbers) * obj.cpus_per_node;           
        end
        
        function num_running_jobs = get.num_running_jobs(obj)
            obj.check_jobs();
            s = size(obj.file_names);
            num_running_jobs = s(1);
        end
              
        function check_jobs(obj)
            if (~isempty(obj.file_names))
                s = size(obj.file_names);
                for i = s(1):-1:1
                    fid = fopen(obj.file_names(i,:));
                    if (fid ~= -1)
                        [obj.file_names index] = setdiff(obj.file_names, obj.file_names(i,:), 'rows');
                        obj.node_of_job = obj.node_of_job(index,:);
                    end
                end
            end
        end
        
        function node_number = free_node(obj)
            for i = 1:length(obj.node_numbers)
                curr_node = char(obj.node_numbers(i));
                count = 0;
                s = size(obj.node_of_job);
                for j = 1:s(1)
                    if (curr_node == obj.node_of_job(j,:))
                        count = count + 1;
                    end
                end
                if (count < obj.cpus_per_node)
                    node_number = curr_node;
                    return;
                end
            end   
            disp('Warning: No free node found');
            node_number = '';
        end
        
        function node_number = register_job(obj, file_name)
            while (obj.num_running_jobs == obj.max_num_jobs)
                pause(5);
            end   
            node_number = obj.free_node();
            obj.file_names = strvcat(obj.file_names, file_name);
            obj.node_of_job = strvcat(obj.node_of_job, node_number);
        end
    end
end