classdef TestSeqs
    properties
        seqs
    end
    properties (Dependent = true)
        num_seqs
        num_sens_seqs
        num_spec_seqs
    end
    
    methods
        function s = TestSeqs(small_subs, large_subs, num_sens_per_sub_pair, num_tests_per_seq_pair, num_spec_per_sub, begin_sens, elig_len_sens, short_segm_len, long_segms_len, begin_spec, elig_len_spec, spec_segm_len, al)
            NO_FIRST_SEQ = 1;
            NO_LAST_SEQ = 1;
            LARGE_PRIME = max(primes(1000));
            counter = 0;
            s.seqs = [];

            % generate seqs for testing specivity
            counter = 0;            
            for i=1:length(large_subs)
                no_seqs = al.no_seqs_of_sub(large_subs{i});
                for j=1:num_spec_per_sub
                    no_seq = no_seqs(mod(j * LARGE_PRIME, length(no_seqs)) + 1);
                    curr_seq = al.seqs(no_seq);
                    if (curr_seq.pos_without_gaps(begin_spec) == 0)
                        disp('Warning: Begin of test sequence set to a value before the sequence begins in the alignment');
                        p = 1 + mod(counter * LARGE_PRIME, elig_len_spec);
                    else
                        p = curr_seq.pos_without_gaps(begin_spec) + mod(counter * LARGE_PRIME, elig_len_spec);
                    end
                    begin_pos = curr_seq.pos_with_gaps(p);  
                    if (p + spec_segm_len - 1 > length(curr_seq.pos_with_gaps))
                        disp('Warning: End of test sequence set to a value behind the sequence ends in the alignment');
                        end_pos = curr_seq.pos_with_gaps(length(curr_seq.pos_with_gaps));
                    else
                        end_pos = curr_seq.pos_with_gaps(p + spec_segm_len - 1);
                    end                   
                    s.seqs = [s.seqs TestSeq(begin_pos, end_pos, [], no_seq, 'spec', al)];
                    counter = counter + 1;
                end
            end
            
           % generate seqs for testing sensitivity
            for i=1:length(large_subs)
                large_sub = large_subs{i};
                no_seqs_large = al.no_seqs_of_sub(large_sub);                
                for j=1:length(small_subs)
                    small_sub = small_subs{j};
                    no_seqs_small = al.no_seqs_of_sub(small_sub);
                    for k=1:num_sens_per_sub_pair                        
                        no_seq_large = no_seqs_large(mod(k * LARGE_PRIME, length(no_seqs_large)) + 1);        
                        no_seq_small = no_seqs_small(mod(k * LARGE_PRIME, length(no_seqs_small)) + 1);
                        seq_large = al.seqs(no_seq_large);
                        seq_small = al.seqs(no_seq_small);
                        
                        begin_small = [];
                        end_small = [];
                        begin_seq = [];
                        end_seq = [];
                        for l=1:num_tests_per_seq_pair                                                        
                            % begin of whole seq                            
                            p = seq_large.pos_without_gaps(begin_sens) + mod(counter * LARGE_PRIME, elig_len_sens);
                            if ~seq_large.is_valid_pos_without_gaps(p), continue, end
                            curr_begin_seq = seq_large.pos_with_gaps(p);   
                            
                            % begin of short segm
                            p = seq_large.pos_without_gaps(curr_begin_seq) + long_segms_len;
                            if ~seq_large.is_valid_pos_without_gaps(p), continue, end
                            curr_begin_small = seq_large.pos_with_gaps(p);     

                            % end of short segm
                            p = seq_small.pos_without_gaps(curr_begin_small) + short_segm_len;
                            if ~seq_small.is_valid_pos_without_gaps(p), continue, end
                            curr_end_small = seq_small.pos_with_gaps(p);     
                                                                                    
                            % end of whole seq
                            p = seq_large.pos_without_gaps(curr_end_small) + long_segms_len;
                            if ~seq_large.is_valid_pos_without_gaps(p), continue, end
                            curr_end_seq = seq_large.pos_with_gaps(p);                            

                            begin_small = [begin_small curr_begin_small];
                            end_small = [end_small curr_end_small];
                            begin_seq = [begin_seq curr_begin_seq];
                            end_seq = [end_seq curr_end_seq];
                            
                            counter = counter + 1;
                        end                                             
                        
                        for l=1:length(begin_small)
                            new_seq = TestSeq(begin_seq(l), end_seq(l), [begin_small(l) end_small(l)], [no_seq_large, no_seq_small, no_seq_large], 'sens', al);
                            s.seqs = [s.seqs new_seq];
                        end
                    end
                end
            end
            
            % remove sequences (for testing purpose)
            len = length(s.seqs);
            s.seqs = s.seqs(min(NO_FIRST_SEQ, len):min(NO_LAST_SEQ, len));
            if (length(s.seqs) < len)
                disp([num2str(len - length(s.seqs)) ' test sequences have been removed'])
            end
            for i=1:length(s.seqs)
                disp(s.seqs(i));
            end
        end
        
        function num_sens_seqs = get.num_sens_seqs(obj)
            num_sens_seqs = 0;
            for i=1:obj.num_seqs
                if (obj.seqs(i).is_sens())
                    num_sens_seqs = num_sens_seqs + 1;
                end
            end
        end
        function num_spec_seqs = get.num_spec_seqs(obj)
            num_spec_seqs = 0;
            for i=1:obj.num_seqs
                if (obj.seqs(i).is_spec())
                    num_spec_seqs = num_spec_seqs + 1;
                end
            end
        end
        function num_seqs = get.num_seqs(obj)
            num_seqs = length(obj.seqs);
        end
        
        function res = score(obj, model_para, al)
            global consts;            
            WAITING_TIME = 1;            
            res = struct('score', 0, 'sens', 0, 'spec', 0, 'orig_segms', 0, 'est_segms', 0, 'seq_scores', 0, 'seq_pos_scores', 0, 'seq_segm_scores', 0);
            
            % submit scoring jobs
            jobs = [];
            for i=1:length(obj.seqs)
                jobs{i} = obj.seqs(i).submit_score_job(model_para, al);
                disp(['job ', num2str(i), ' submitted']);
            end
            
            % wait until all jobs are finished
            all_finished = false;
            finished = zeros(1, length(obj.seqs));
            while (~all_finished)
                b = true;
                for i=1:length(obj.seqs)
                    if jobs{i}.is_finished()
                        if (~finished(i))
                            finished(i) = true;
                            disp(['Job ' num2str(i) ' finished, ' num2str(sum(finished)) ' of ' num2str(length(obj.seqs)) ' jobs finished'])
                        end
                    else
                        b = false;
                    end
                end
                all_finished = b;
                if ~all_finished, pause(WAITING_TIME), end
            end
            disp(char(10))
            
            % get scores
            spec = 0;
            sens = 0;
            res.orig_segms = [];
            res.est_segms = [];
            res.seq_scores = [];
            res.seq_pos_scores = [];
            res.seq_segm_scores = [];
            if (consts.save_to_db)
                 DB_con = SQL.get_connection(); 
            end
            for i=1:length(obj.seqs)
                est_segm = JPHMM.results(jobs{i}.stamp, obj.seqs(i).begin_pos, obj.seqs(i).end_pos, obj.seqs(i));           
                res_curr_scoring = obj.seqs(i).segm.score(est_segm, obj.seqs(i));
                curr_score = res_curr_scoring.score;
                if (obj.seqs(i).is_spec)                    
                    spec = spec + curr_score / obj.num_spec_seqs;
                else
                    sens = sens + curr_score / obj.num_sens_seqs;
                end
                res.orig_segms = [res.orig_segms obj.seqs(i).segm];
                res.est_segms = [res.est_segms est_segm];
                res.seq_scores = [res.seq_scores curr_score];
                res.seq_pos_scores = [res.seq_pos_scores res_curr_scoring.pos_score];
                res.seq_segm_scores = [res.seq_segm_scores res_curr_scoring.segm_score];
                
                if (consts.save_to_db)                   
                    % retrieve or save all data in DB
                    bps_orig_id = SQL.save_vector([obj.seqs(i).segm.begin_pos obj.seqs(i).segm.bps obj.seqs(i).segm.end_pos], 'BPs', 'BP');
                    subs_orig_id = SQL.save_vector(obj.seqs(i).segm.subs, 'Subs', 'Sub');

                    v = cell(1, length(obj.seqs(i).no_used_seqs));
                    for j=1:length(obj.seqs(i).no_used_seqs)
                        v{j} = al.seqs(obj.seqs(i).no_used_seqs(j)).name;
                    end
                    used_seqs_id = SQL.save_vector(v, 'Used_Seqs', 'Seq');

                    bps_est_id = SQL.save_vector([est_segm.begin_pos est_segm.bps est_segm.end_pos], 'BPs', 'BP');

                    subs_est_id = SQL.save_vector(est_segm.subs, 'Subs', 'Sub');

                    % save results to DB 
                    if (~strcmp(consts.strategy, 'ml'))
                        sql_stat = ['REPLACE INTO Result VALUES (' num2str(bps_orig_id) ', ' num2str(subs_orig_id) ', ' num2str(used_seqs_id) ', ' ...
                                     '''' consts.strategy ''', ' num2str(model_para.jp) ', ' num2str(model_para.mu) ', 0, 0, 0, ' num2str(bps_est_id) ', ' num2str(subs_est_id) ', '  ...
                                     num2str(curr_score) ', ' num2str(res_curr_scoring.pos_score) ', ' num2str(res_curr_scoring.segm_score) ', NULL)'];                
                    else
                        sql_stat = ['REPLACE INTO Result VALUES (' num2str(bps_orig_id) ', ' num2str(subs_orig_id) ', ' num2str(used_seqs_id) ', ' ...
                                     '''' consts.strategy ''', ' num2str(model_para.jp) ', 1, ' num2str(model_para.lambda) ', ' num2str(model_para.gamma_div_lambda) ', ' ...
                                     num2str(model_para.res_per_sector) ', ' num2str(bps_est_id) ', ' num2str(subs_est_id) ', '  ...
                                     num2str(curr_score) ', ' num2str(res_curr_scoring.pos_score) ', ' num2str(res_curr_scoring.segm_score) ', NULL)'];                
                    end
                    DB_con.executeSQLStatement(sql_stat);    
                end
            end  
            
            if (~consts.save_to_db)
                disp('not saving to SQL DB');
            end
            
            if ~obj.num_sens_seqs
                res.score = spec;
            else
                if ~obj.num_spec_seqs
                    res.score = sens;
                else
                    res.score = 2 / (1 / spec + 1 / sens);
                end
            end
            res.spec = spec;
            res.sens = sens;
        end
    end
end